@@ -1,6 +1,7 @@
 /* fish.c:
  *
  * Copyright (C) 1998-2002 Free Software Foundation, Inc.
+ * Copyright (C) 2002-2005 Vincent Untz
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,23 +21,23 @@
  * Authors:
  *      George Lebl  <jirka@5z.com>
  *      Mark McLoughlin <mark@skynet.ie>
+ *      Vincent Untz <vuntz@gnome.org>
  */
 
 #include <config.h>
 #include <string.h>
 #include <time.h>
 
+#include <math.h>
+#include <cairo.h>
 #include <glib-object.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <panel-applet.h>
 #include <panel-applet-gconf.h>
 #include <gconf/gconf-client.h>
-#include <libgnomeui/gnome-file-entry.h>
 #include <libgnomeui/gnome-help.h>
-#include <libgnomeui/gnome-pixmap-entry.h>
 #include <libgnomeui/gnome-uidefs.h>
-#include <libart_lgpl/libart.h>
 #include <glade/glade-xml.h>
 
 #define FISH_APPLET(o) (G_TYPE_CHECK_INSTANCE_CAST ((o), \
@@ -45,6 +46,8 @@
 #define FISH_IS_APPLET(o) (G_TYPE_CHECK_INSTANCE_TYPE ((o), \
 			   FISH_TYPE_APPLET))
 
+#define FISH_ICON "gnome-panel-fish"
+
 #define N_FISH_PREFS 6
 
 #define LOCKDOWN_COMMANDLINE_KEY "/desktop/gnome/lockdown/disable_command_line"
@@ -80,10 +83,10 @@ typedef struct {
 
 	GtkWidget         *preferences_dialog;
 	GtkWidget         *name_entry;
-	GtkWidget         *pixmap_entry;
-	GtkWidget         *image_entry;
 	GtkWidget         *command_label;
 	GtkWidget         *command_entry;
+	GtkWidget         *preview_image;
+	GtkWidget         *image_chooser;
 	GtkWidget         *frames_spin;
 	GtkWidget         *speed_spin;
 	GtkWidget         *rotate_toggle;
@@ -123,6 +126,19 @@ static int fools_month      = 0;
 static int fools_hour_start = 0;
 static int fools_hour_end   = 0;
 
+static char *
+get_image_path (FishApplet *fish)
+{
+	char *path;
+
+	if (g_path_is_absolute (fish->image))
+		path = g_strdup (fish->image);
+	else
+		path = g_strdup_printf ("%s/%s", FISH_ICONDIR, fish->image);
+
+	return path;
+}
+
 static void
 show_help (FishApplet *fish, const char *link_id)
 {
@@ -148,6 +164,7 @@ show_help (FishApplet *fish, const char *link_id)
 				  G_CALLBACK (gtk_widget_destroy),
 				  NULL);
 
+		gtk_window_set_icon_name (GTK_WINDOW (dialog), FISH_ICON);
 		gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 		gtk_window_set_screen (GTK_WINDOW (dialog),
 				       gtk_widget_get_screen (GTK_WIDGET (fish)));
@@ -172,18 +189,36 @@ name_value_changed (GtkEntry   *entry,
 }
 
 static void
-image_value_changed (GtkEntry   *entry,
-		     FishApplet *fish)
+image_value_changed (GtkFileChooser *chooser,
+		     FishApplet     *fish)
 {	char *path;
+	char *image;
+	char *path_gconf;
 
-	path = gnome_file_entry_get_full_path (
-			GNOME_FILE_ENTRY (fish->pixmap_entry), TRUE);
-	if (!path || !path [0]) {
+	path = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
+
+	if (!path || !path[0]) {
 		g_free (path);
 		return;
 	}
 
-	panel_applet_gconf_set_string (PANEL_APPLET (fish), "image", path, NULL);
+	path_gconf = get_image_path (fish);
+	if (!strcmp (path, path_gconf)) {
+		g_free (path);
+		g_free (path_gconf);
+		return;
+	}
+	g_free (path_gconf);
+
+	if (!strncmp (path, FISH_ICONDIR, strlen (FISH_ICONDIR))) {
+		image = path + strlen (FISH_ICONDIR);
+		while (*image && *image == G_DIR_SEPARATOR)
+			image++;
+	} else
+		image = path;
+
+	panel_applet_gconf_set_string (PANEL_APPLET (fish), "image",
+				       image, NULL);
 
 	g_free (path);
 }
@@ -321,13 +356,43 @@ setup_sensitivity (FishApplet *fish,
 
 }
 
+static void
+chooser_preview_update (GtkFileChooser *file_chooser,
+			gpointer data)
+{
+	GtkWidget *preview;
+	char      *filename;
+	GdkPixbuf *pixbuf;
+	gboolean   have_preview;
+
+	preview = GTK_WIDGET (data);
+	filename = gtk_file_chooser_get_preview_filename (file_chooser);
+
+	if (filename == NULL)
+		return;
+
+	pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 128, 128, NULL);
+	have_preview = (pixbuf != NULL);
+	g_free (filename);
+
+	gtk_image_set_from_pixbuf (GTK_IMAGE (preview), pixbuf);
+	if (pixbuf)
+		g_object_unref (pixbuf);
+
+	gtk_file_chooser_set_preview_widget_active (file_chooser,
+						    have_preview);
+}
+
 static void 
 display_preferences_dialog (BonoboUIComponent *uic,
 			    FishApplet        *fish,
 			    const char        *verbname)
 {
-	GladeXML  *xml;
-	GtkWidget *button;
+	GladeXML      *xml;
+	GtkWidget     *button;
+	GtkFileFilter *filter;
+	GtkWidget     *chooser_preview;
+	char          *path;
 
 	if (fish->preferences_dialog) {
 		gtk_window_set_screen (GTK_WINDOW (fish->preferences_dialog),
@@ -344,6 +409,8 @@ display_preferences_dialog (BonoboUIComponent *uic,
 
 	gtk_window_set_wmclass (GTK_WINDOW (fish->preferences_dialog),
 				"fish", "Fish");
+	gtk_window_set_icon_name (GTK_WINDOW (fish->preferences_dialog),
+				  FISH_ICON);
 	gtk_dialog_set_default_response (
 		GTK_DIALOG (fish->preferences_dialog), GTK_RESPONSE_OK);
 
@@ -359,18 +426,34 @@ display_preferences_dialog (BonoboUIComponent *uic,
 			   NULL /* label_post */,
 			   "name" /* key */);
 
-	fish->pixmap_entry = glade_xml_get_widget (xml, "image_entry");
-	gnome_pixmap_entry_set_pixmap_subdir (GNOME_PIXMAP_ENTRY (fish->pixmap_entry),
-					    FISH_ICONDIR);
-	fish->image_entry = gnome_file_entry_gtk_entry (
-				GNOME_FILE_ENTRY (fish->pixmap_entry));
-	gtk_entry_set_text (GTK_ENTRY (fish->image_entry), fish->image);
+	fish->preview_image = glade_xml_get_widget (xml, "preview_image");
+	if (fish->pixbuf)
+		gtk_image_set_from_pixbuf (GTK_IMAGE (fish->preview_image),
+					   fish->pixbuf);
 
-	g_signal_connect (fish->image_entry, "changed",
+	fish->image_chooser =  glade_xml_get_widget (xml, "image_chooser");
+	filter = gtk_file_filter_new ();
+	gtk_file_filter_set_name (filter, _("Images"));
+	gtk_file_filter_add_pixbuf_formats (filter);
+	gtk_file_chooser_add_filter (GTK_FILE_CHOOSER (fish->image_chooser),
+				     filter);
+	gtk_file_chooser_set_filter (GTK_FILE_CHOOSER (fish->image_chooser),
+				     filter);
+	chooser_preview = gtk_image_new ();
+	gtk_file_chooser_set_preview_widget (GTK_FILE_CHOOSER (fish->image_chooser),
+					     chooser_preview);
+	g_signal_connect (fish->image_chooser, "update-preview",
+			  G_CALLBACK (chooser_preview_update), chooser_preview);
+	path = get_image_path (fish);
+	gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (fish->image_chooser),
+				       path);
+	g_free (path);
+
+	g_signal_connect (fish->image_chooser, "selection-changed",
 			  G_CALLBACK (image_value_changed), fish);
 
 	setup_sensitivity (fish, xml,
-			   "image_entry" /* wid */,
+			   "image_chooser" /* wid */,
 			   "image_label" /* label */,
 			   NULL /* label_post */,
 			   "image" /* key */);
@@ -468,9 +551,9 @@ display_about_dialog (BonoboUIComponent *uic,
 	const char *about_format = _("%s has no use what-so-ever. "
 				     "It only takes up disk space and "
 				     "compilation time, and if loaded it also "
-				     "takes up precious panel space and memory. "
-				     "If anyone is found using it, he "
-				     "should be promptly sent for a psychiatric "
+				     "takes up precious panel space and "
+				     "memory. Anybody found using it should be "
+				     "promptly sent for a psychiatric "
 				     "evaluation.");
 	const char  *documenters [] = {
 		"Telsa Gwynne <hobbit@aloss.ukuu.org.uk>",
@@ -503,12 +586,13 @@ display_about_dialog (BonoboUIComponent *uic,
 		      "authors", (const char **) authors,
 		      "documenters", documenters,
 		      "translator-credits", _("translator-credits"),
-		      "logo-icon-name", "gnome-fish",
+		      "logo-icon-name", FISH_ICON,
 		      NULL);
 
 	g_free (descr);
 	g_free (authors [0]);
 
+	gtk_window_set_icon_name (GTK_WINDOW (fish->about_dialog), FISH_ICON);
 	gtk_window_set_wmclass (
 		GTK_WINDOW (fish->about_dialog), "fish", "Fish");
 	gtk_window_set_screen (GTK_WINDOW (fish->about_dialog),
@@ -517,6 +601,11 @@ display_about_dialog (BonoboUIComponent *uic,
 	g_signal_connect (fish->about_dialog, "destroy",
 			  G_CALLBACK (gtk_widget_destroyed),
 			  &fish->about_dialog);
+
+	g_signal_connect (fish->about_dialog, "response",
+			  G_CALLBACK (gtk_widget_destroy),
+			  NULL);
+
 	gtk_widget_show (fish->about_dialog);
 }
 
@@ -559,6 +648,7 @@ something_fishy_going_on (FishApplet *fish,
 			  G_CALLBACK (gtk_widget_destroy),
 			  NULL);
 
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), FISH_ICON);
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	gtk_window_set_screen (GTK_WINDOW (dialog),
 			       gtk_widget_get_screen (GTK_WIDGET (fish)));
@@ -782,6 +872,8 @@ display_fortune_dialog (FishApplet *fish)
 				GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 				NULL);
 
+		gtk_window_set_icon_name (GTK_WINDOW (fish->fortune_dialog),
+					  FISH_ICON);
 		gtk_dialog_set_has_separator (
 			GTK_DIALOG (fish->fortune_dialog), FALSE);
 
@@ -970,9 +1062,19 @@ image_changed_notify (GConfClient *client,
 	load_fish_image (fish);
 	update_pixmap (fish);
 
-	if (fish->image_entry &&
-	    strcmp (gtk_entry_get_text (GTK_ENTRY (fish->image_entry)), fish->image))
-		gtk_entry_set_text (GTK_ENTRY (fish->image_entry), fish->image);
+	if (fish->image_chooser) {
+		char *path_gconf;
+		char *path_chooser;
+
+		path_gconf = get_image_path (fish);
+		path_chooser = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (fish->image_chooser));
+		if (strcmp (path_gconf, path_chooser))
+			gtk_file_chooser_set_filename (GTK_FILE_CHOOSER (fish->image_chooser),
+						       path_gconf);
+
+		g_free (path_gconf);
+		g_free (path_chooser);
+	}
 }
 
 static void
@@ -1288,10 +1390,7 @@ load_fish_image (FishApplet *fish)
 	if (!fish->image)
 		return FALSE;
 
-	if (g_path_is_absolute (fish->image))
-		path = g_strdup (fish->image);
-	else
-		path = g_strdup_printf ("%s/%s", FISH_ICONDIR, fish->image);
+	path = get_image_path (fish);
 
 	pixbuf = gdk_pixbuf_new_from_file (path, &error);
 	if (error) {
@@ -1305,6 +1404,10 @@ load_fish_image (FishApplet *fish)
 		g_object_unref (fish->pixbuf);
 	fish->pixbuf = pixbuf;
 
+	if (fish->preview_image)
+		gtk_image_set_from_pixbuf (GTK_IMAGE (fish->preview_image),
+					   fish->pixbuf);
+
 	g_free (path);
 
 	return TRUE;
@@ -1314,52 +1417,45 @@ static void
 update_pixmap (FishApplet *fish)
 {
 	GtkWidget *widget = fish->drawing_area;
-	GdkGC     *gc;
 	int        width  = -1;
 	int        height = -1;
 	int        pixbuf_width = -1;
 	int        pixbuf_height = -1;
 	gboolean   rotate = FALSE;
-	double     affine [6];
-	guchar    *rgb;
+	cairo_t   *cr;
+	cairo_matrix_t matrix;
+	cairo_pattern_t *pattern;
 
 	if (!GTK_WIDGET_REALIZED (widget) ||
 	    widget->allocation.width <= 0 ||
 	    widget->allocation.height <= 0)
 		return;
 
+	if (!fish->pixbuf && !load_fish_image (fish))
+		return;
+
 	if (fish->rotate &&
 	    (fish->orientation == PANEL_APPLET_ORIENT_LEFT ||
 	     fish->orientation == PANEL_APPLET_ORIENT_RIGHT))
 		rotate = TRUE;
 
-	if (fish->pixbuf || load_fish_image (fish)) {
-		pixbuf_width  = gdk_pixbuf_get_width  (fish->pixbuf);
-		pixbuf_height = gdk_pixbuf_get_height (fish->pixbuf);
+	pixbuf_width  = gdk_pixbuf_get_width  (fish->pixbuf);
+	pixbuf_height = gdk_pixbuf_get_height (fish->pixbuf);
 
-		if (fish->orientation == PANEL_APPLET_ORIENT_UP ||
-		    fish->orientation == PANEL_APPLET_ORIENT_DOWN) {
-			height = widget->allocation.height;
-			width  = pixbuf_width * ((gdouble) height / pixbuf_height);
-			widget->requisition.width = width / fish->n_frames;
-		} else {
-			if (!rotate) {
-				width = widget->allocation.width * fish->n_frames;
-				height = pixbuf_height * ((gdouble) width / pixbuf_width);
-				widget->requisition.height = height;
-			} else {
-				width = widget->allocation.width;
-				height = pixbuf_width * ((gdouble) width / pixbuf_height);
-				widget->requisition.height = height / fish->n_frames;
-			}
-		}
+	if (fish->orientation == PANEL_APPLET_ORIENT_UP ||
+	    fish->orientation == PANEL_APPLET_ORIENT_DOWN) {
+		height = widget->allocation.height;
+		width  = pixbuf_width * ((gdouble) height / pixbuf_height);
+		widget->requisition.width = width / fish->n_frames;
 	} else {
-		if (rotate) {
-			width  = widget->allocation.width;
-			height = widget->allocation.height * fish->n_frames;
+		if (!rotate) {
+			width = widget->allocation.width * fish->n_frames;
+			height = pixbuf_height * ((gdouble) width / pixbuf_width);
+			widget->requisition.height = height;
 		} else {
-			height = widget->allocation.height;
-			width  = widget->allocation.width * fish->n_frames;
+			width = widget->allocation.width;
+			height = pixbuf_width * ((gdouble) width / pixbuf_height);
+			widget->requisition.height = height / fish->n_frames;
 		}
 	}
 
@@ -1372,71 +1468,55 @@ update_pixmap (FishApplet *fish)
 		g_object_unref (fish->pixmap);
 	fish->pixmap = gdk_pixmap_new (widget->window, width, height, -1);
 
-	if (!fish->pixbuf)
-		return;
-
 	gtk_widget_queue_resize (widget);
 
 	g_assert (pixbuf_width != -1 && pixbuf_height != -1);
 
-	affine [1] = affine [2] = affine [4] = affine [5] = 0;
+	cr = gdk_cairo_create (fish->pixmap);
 
-	if (!rotate) {
-		affine [0] = width  / (double) pixbuf_width;
-		affine [3] = height / (double) pixbuf_height;
-	} else {
-		double tmp [6];
+	cairo_set_source_rgb (cr, 1, 1, 1);
+	cairo_paint (cr);
 
-		affine [0] = height / (double) pixbuf_width;
-		affine [3] = width  / (double) pixbuf_height;
-		
-		art_affine_rotate (tmp, 270);
-		art_affine_multiply (affine, affine, tmp);
-                art_affine_translate (tmp, 0, height);
-                art_affine_multiply (affine, affine, tmp);
-	}
+	gdk_cairo_set_source_pixbuf (cr, fish->pixbuf, 0, 0);
+	pattern = cairo_get_source (cr);
+	cairo_pattern_set_filter (pattern, CAIRO_FILTER_BEST);
+
+	cairo_matrix_init_identity (&matrix);
 
 	if (fish->april_fools) {
-                double tmp [6];
+		cairo_matrix_translate (&matrix,
+					pixbuf_width - 1, pixbuf_height - 1);
+		cairo_matrix_rotate (&matrix, M_PI);
+	}
 
-                art_affine_rotate (tmp, 180);
-                art_affine_multiply (affine, affine, tmp);
-                art_affine_translate (tmp, width, height);
-                art_affine_multiply (affine, affine, tmp);
-        }
+	if (rotate) {
+		if (fish->orientation == PANEL_APPLET_ORIENT_RIGHT) {
+			cairo_matrix_translate (&matrix, pixbuf_width - 1, 0);
+			cairo_matrix_rotate (&matrix, M_PI * 0.5);
+		} else {
+			cairo_matrix_translate (&matrix, 0, pixbuf_height - 1);
+			cairo_matrix_rotate (&matrix, M_PI * 1.5);
+		}
+		cairo_matrix_scale (&matrix,
+				    (double) (pixbuf_height - 1) / width,
+				    (double) (pixbuf_width - 1) / height);
+	} else {
+		cairo_matrix_scale (&matrix,
+				    (double) (pixbuf_width - 1) / width,
+				    (double) (pixbuf_height - 1) / height);
+	}
 
-	rgb = g_new0 (guchar, width * height * 3);
+	cairo_pattern_set_matrix (pattern, &matrix);
 
-	if (gdk_pixbuf_get_has_alpha (fish->pixbuf))
-		art_rgb_rgba_affine (rgb, 0, 0, width, height, width * 3,
-				     gdk_pixbuf_get_pixels (fish->pixbuf),
-                                     pixbuf_width, pixbuf_height,
-                                     gdk_pixbuf_get_rowstride (fish->pixbuf),
-                                     affine,
-                                     ART_FILTER_NEAREST,
-                                     NULL);
-        else
-                art_rgb_affine (rgb, 0, 0, width, height, width * 3,
-                                gdk_pixbuf_get_pixels (fish->pixbuf),
-                                pixbuf_width, pixbuf_height,
-                                gdk_pixbuf_get_rowstride (fish->pixbuf),
-                                affine,
-                                ART_FILTER_NEAREST,
-                                NULL);
+	cairo_rectangle (cr, 0, 0, width, height);
+	cairo_fill (cr);
 
+	if (fish->april_fools) {
+		cairo_set_source_rgb (cr, 1, 0.5, 0);
+		cairo_paint_with_alpha (cr, 0.25);
+	}
 
-	if (fish->april_fools)
-                art_rgb_run_alpha (rgb, 255, 128, 0, 70, width * height);
-
-	gc = gdk_gc_new (fish->pixmap);
-
-	gdk_draw_rgb_image (fish->pixmap, gc,
-			    0, 0, width, height,
-                            GDK_RGB_DITHER_NORMAL,
-                            rgb, width * 3);
-
-	g_object_unref (gc);
-        g_free (rgb);
+	cairo_destroy (cr);
 }
 
 static gboolean
@@ -1456,11 +1536,14 @@ fish_applet_expose_event (GtkWidget      *widget,
 	src_x = event->area.x;
 	src_y = event->area.y;
 
-	if (fish->rotate &&
-	    (fish->orientation == PANEL_APPLET_ORIENT_LEFT ||
-	     fish->orientation == PANEL_APPLET_ORIENT_RIGHT))
-		src_y += ((height * fish->current_frame) / fish->n_frames);
-	else
+	if (fish->rotate) {
+		if (fish->orientation == PANEL_APPLET_ORIENT_RIGHT)
+			src_y += ((height * (fish->n_frames - 1 - fish->current_frame)) / fish->n_frames);
+		else if (fish->orientation == PANEL_APPLET_ORIENT_LEFT)
+			src_y += ((height * fish->current_frame) / fish->n_frames);
+		else
+			src_x += ((width * fish->current_frame) / fish->n_frames);
+	} else
 		src_x += ((width * fish->current_frame) / fish->n_frames);
 
 	gdk_draw_drawable (widget->window,
@@ -1517,42 +1600,6 @@ fish_applet_change_orient (PanelApplet       *applet,
 		update_pixmap (fish);
 }
 
-static void
-fish_applet_change_background (PanelApplet               *panel_applet,
-			       PanelAppletBackgroundType  type,
-			       GdkColor                  *color,
-			       GdkPixmap                 *pixmap)
-{
-	FishApplet *applet;
-	GtkRcStyle *rc_style;
-	GtkStyle   *style;
-
-	applet = (FishApplet *) panel_applet;
-
-	/* reset style */
-	gtk_widget_set_style (GTK_WIDGET (applet), NULL);
-	rc_style = gtk_rc_style_new ();
-	gtk_widget_modify_style (GTK_WIDGET (applet), rc_style);
-	gtk_rc_style_unref (rc_style);
-
-	switch (type) {
-	case PANEL_NO_BACKGROUND:
-		break;
-	case PANEL_COLOR_BACKGROUND:
-		gtk_widget_modify_bg (GTK_WIDGET (applet),
-				      GTK_STATE_NORMAL, color);
-		break;
-	case PANEL_PIXMAP_BACKGROUND:
-		style = gtk_style_copy (GTK_WIDGET (applet)->style);
-		if (style->bg_pixmap[GTK_STATE_NORMAL])
-			g_object_unref (style->bg_pixmap[GTK_STATE_NORMAL]);
-		style->bg_pixmap[GTK_STATE_NORMAL] = g_object_ref (pixmap);
-		gtk_widget_set_style (GTK_WIDGET (applet), style);
-		g_object_unref (style);
-		break;
-	}
-}
-
 static void
 change_water (FishApplet *fish)
 {
@@ -1561,8 +1608,10 @@ change_water (FishApplet *fish)
 	dialog = gtk_message_dialog_new (
 			NULL, 0, GTK_MESSAGE_INFO,
 			GTK_BUTTONS_OK,
-			_("The water needs changing!\n"
-			  "(Look at today's date)"));
+			_("The water needs changing"));
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  _("Look at today's date!"));
+	gtk_window_set_icon_name (GTK_WINDOW (dialog), FISH_ICON);
 	gtk_window_set_wmclass (GTK_WINDOW (dialog), "fish", "Fish");
 	gtk_window_set_screen (GTK_WINDOW (dialog),
 			       gtk_widget_get_screen (GTK_WIDGET (fish)));
@@ -1654,36 +1703,14 @@ handle_button_release (FishApplet     *fish,
 static void
 set_tooltip (FishApplet *fish)
 {
-	GtkTooltips *tooltips;
-	const char  *desc_format = _("%s the Fish, the fortune teller");
-	char        *desc;
+	const char *desc_format = _("%s the Fish, the fortune teller");
+	char       *desc;
 
-	tooltips = g_object_get_data (G_OBJECT (fish), "tooltips");
-	if (!tooltips) {
-		tooltips = gtk_tooltips_new ();
-		g_object_ref (tooltips);
-		gtk_object_sink (GTK_OBJECT (tooltips));
-
-		g_object_set_data (G_OBJECT (fish), "tooltips", tooltips);
-	}
-
-	desc = g_strdup_printf (desc_format, fish->name);
-	gtk_tooltips_set_tip (tooltips, GTK_WIDGET (fish), desc, NULL);
+	desc = g_markup_printf_escaped (desc_format, fish->name);
+	g_object_set (fish, "tooltip-markup", desc, NULL);
 	g_free (desc);
 }
 
-static void
-destroy_tooltip (FishApplet *fish)
-{
-	GtkTooltips *tooltips;
-
-	tooltips = g_object_get_data (G_OBJECT (fish), "tooltips");
-	if (tooltips) {
-		g_object_unref (tooltips);
-		g_object_set_data (G_OBJECT (fish), "tooltips", NULL);
-	}
-}
-
 static void
 setup_fish_widget (FishApplet *fish)
 {
@@ -1842,7 +1869,9 @@ fish_applet_fill (FishApplet *fish)
 					      NULL);
 	}
 
-	gtk_window_set_default_icon_name ("gnome-fish");
+#ifndef FISH_INPROCESS
+	gtk_window_set_default_icon_name (FISH_ICON);
+#endif
 	setup_fish_widget (fish);
 
 	return TRUE;
@@ -1920,8 +1949,6 @@ fish_applet_destroy (GtkObject *object)
 
 	fish_close_channel (fish);
 
-	destroy_tooltip (fish);
-
 	GTK_OBJECT_CLASS (parent_class)->destroy (object);
 }
 
@@ -1960,10 +1987,10 @@ fish_applet_instance_init (FishApplet      *fish,
 
 	fish->preferences_dialog = NULL;
 	fish->name_entry         = NULL;
-	fish->pixmap_entry       = NULL;
-	fish->image_entry        = NULL;
 	fish->command_label      = NULL;
 	fish->command_entry      = NULL;
+	fish->preview_image      = NULL;
+	fish->image_chooser      = NULL;
 	fish->frames_spin        = NULL;
 	fish->speed_spin         = NULL;
 	fish->rotate_toggle      = NULL;
@@ -1984,6 +2011,9 @@ fish_applet_instance_init (FishApplet      *fish,
 
 	panel_applet_set_flags (PANEL_APPLET (fish),
 				PANEL_APPLET_EXPAND_MINOR);
+
+	panel_applet_set_background_widget (PANEL_APPLET (fish),
+					    GTK_WIDGET (fish));
 }
 
 static void
@@ -1995,7 +2025,6 @@ fish_applet_class_init (FishAppletClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	applet_class->change_orient = fish_applet_change_orient;
-	applet_class->change_background = fish_applet_change_background;
 
 	gtkobject_class->destroy = fish_applet_destroy;
 
@@ -2026,9 +2055,18 @@ fish_applet_get_type (void)
 	return type;
 }
 
+#ifdef FISH_INPROCESS
+PANEL_APPLET_BONOBO_SHLIB_FACTORY ("OAFIID:GNOME_FishApplet_Factory",
+				   fish_applet_get_type (),
+				   "That-stupid-fish",
+				   fishy_factory,
+				   NULL)
+
+#else
 PANEL_APPLET_BONOBO_FACTORY ("OAFIID:GNOME_FishApplet_Factory",
 			     fish_applet_get_type (),
 			     "That-stupid-fish",
 			     "0",
 			     fishy_factory,
 			     NULL)
+#endif
