@@ -9,6 +9,7 @@
 
 #include "panel-ditem-editor.h"
 #include "panel-util.h"
+#include "panel-icon-names.h"
 
 #include "nothing.cP"
 
@@ -74,50 +75,6 @@ validate_for_filename (char *file)
 	}
 }
 
-static char *
-get_unique_name (const char *dir,
-		 const char *name)
-{
-	int i;
-	char *full;
-	char *nameext = g_strdup_printf ("%s.desktop", name);
-
-	full = g_build_path ("/", dir, nameext, NULL);
-	if (!panel_uri_exists (full)) {
-		g_free (nameext);
-		return full;
-	}
-	g_free (full);
-
-	i = 2;
-	for (;;) {
-		g_free (nameext);
-		nameext = g_strdup_printf ("%s%d.desktop", name, i++);
-
-		/* randomize further same name desktops */
-		if (i > 5)
-			i = g_random_int ();
-
-		full = g_build_path ("/", dir, nameext, NULL);
-		if (!panel_uri_exists (full)) {
-			g_free (nameext);
-			return full;
-		}
-		g_free (full);
-	}
-}
-
-static char *
-find_uri_on_save_directory (PanelDItemEditor *dialog,
-			    gpointer          data)
-{
-	char *filename;
-
-	filename = g_object_get_data (G_OBJECT (dialog), "filename");
-
-	return g_strdup (filename);
-}
-
 static char *
 find_uri_on_save (PanelDItemEditor *dialog,
 		  gpointer          data)
@@ -139,7 +96,7 @@ find_uri_on_save (PanelDItemEditor *dialog,
 		filename = g_strdup ("foo");
 
 	dir = g_object_get_data (G_OBJECT (dialog), "dir");
-	uri = get_unique_name (dir, filename);
+	uri = panel_make_unique_desktop_path_from_name (dir, filename);
 
 	g_free (filename);
 
@@ -179,7 +136,7 @@ main (int argc, char * argv[])
 				      GNOME_PARAM_GOPTION_CONTEXT, context,
 				      NULL);
 
-	gtk_window_set_default_icon_name ("launcher-program");
+	gtk_window_set_default_icon_name (PANEL_ICON_LAUNCHER);
 
 	if (desktops == NULL ||
 	    desktops[0] == NULL) {
@@ -223,8 +180,10 @@ main (int argc, char * argv[])
 			} else if (info->type == GNOME_VFS_FILE_TYPE_REGULAR
 				   && g_str_has_suffix (desktops [i], ".directory")
 				   && !create_new) {
-				dlg = panel_ditem_editor_new (NULL, NULL, uri,
-							      _("Directory Properties"));
+				dlg = panel_ditem_editor_new_directory (NULL,
+									NULL,
+									uri,
+									_("Directory Properties"));
 			} else if (info->type == GNOME_VFS_FILE_TYPE_REGULAR
 				   && g_str_has_suffix (desktops [i], ".desktop")
 				   && !create_new) {
@@ -236,47 +195,26 @@ main (int argc, char * argv[])
 					    "already exists\n", uri);
 			} else {
 				g_printerr ("gnome-desktop-item-edit: %s "
-					    "doesn't seem like a desktop "
+					    "does not look like a desktop "
 					    "item\n", uri);
 			}
 
-		} else if (g_str_has_suffix (desktops [i], ".directory")
-			   && !create_new) {
+		} else if (g_str_has_suffix (desktops [i], ".directory")) {
 			/* a non-existant file.  Well we can still edit that
 			 * sort of.  We will just create it new */
-			GKeyFile *key_file;
-			char     *dirname;
-			char     *filename;
+			dlg = panel_ditem_editor_new_directory (NULL, NULL, uri,
+								_("Directory Properties"));
 
-			dirname = g_path_get_dirname (uri);
-			filename = g_strconcat (dirname, "/", ".directory",
-						NULL);
-			g_free (dirname);
-
-			key_file = panel_util_key_file_new_desktop ();
-			panel_util_key_file_set_string (key_file,
-							"Type", "Directory");
-
-			dlg = panel_ditem_editor_new (NULL, key_file, NULL,
-						      _("Directory Properties"));
-			g_object_set_data_full (G_OBJECT (dlg), "filename",
-						filename,
-						(GDestroyNotify)g_free);
-
-			panel_ditem_register_save_uri_func (PANEL_DITEM_EDITOR (dlg),
-							    find_uri_on_save_directory,
-							    NULL);
-
-		} else if (g_str_has_suffix (desktops [i], ".desktop")
-			   && !create_new) {
+		} else if (g_str_has_suffix (desktops [i], ".desktop")) {
 			/* a non-existant file.  Well we can still edit that
 			 * sort of.  We will just create it new */
 			dlg = panel_ditem_editor_new (NULL, NULL, uri,
 						      _("Create Launcher"));
 
 		} else {
-			g_printerr ("gnome-desktop-item-edit: %s does "
-				    "not exist\n", uri);
+			g_printerr ("gnome-desktop-item-edit: %s does not "
+				    "have a .desktop or .directory "
+				    "suffix\n", uri);
 		}
 
 		if (dlg != NULL) {
