@@ -362,7 +362,7 @@ is_this_drop_ok (GtkWidget      *widget,
 		return FALSE;
 
 	if (!text_uri_list)
-		text_uri_list = gdk_atom_intern ("text/uri-list", FALSE);
+		text_uri_list = gdk_atom_intern_static_string ("text/uri-list");
 
 	for (l = context->targets; l; l = l->next) {
 		if (GDK_POINTER_TO_ATOM (l->data) == text_uri_list)
@@ -414,7 +414,7 @@ drag_drop_cb (GtkWidget	        *widget,
 		return FALSE;
 
 	if (text_uri_list == 0)
-		text_uri_list = gdk_atom_intern ("text/uri-list", FALSE);
+		text_uri_list = gdk_atom_intern_static_string ("text/uri-list");
 
 	gtk_drag_get_data (widget, context, text_uri_list, time);
 
