@@ -16,6 +16,7 @@
 
 #include "panel-util.h"
 
+#include <dirent.h>
 #include <string.h>
 #include <unistd.h>
 #include <sys/types.h>
@@ -23,6 +24,7 @@
 #include <glib/gi18n.h>
 #include <libgnome/gnome-desktop-item.h>
 #include <libgnome/gnome-util.h>
+#include <libgnomeui/gnome-icon-lookup.h>
 #include <libgnomeui/gnome-help.h>
 
 #include <libgnomevfs/gnome-vfs-utils.h>
@@ -34,6 +36,7 @@
 #include "xstuff.h"
 #include "panel-globals.h"
 #include "launcher.h"
+#include "panel-icon-names.h"
 
 static int
 panel_ditem_launch (GnomeDesktopItem  *item,
@@ -140,6 +143,37 @@ panel_launch_desktop_file (const char  *desktop_file,
 	}
 }
 
+char *
+panel_util_make_exec_uri_for_desktop (const char *exec)
+{
+	GString    *str;
+	const char *c;
+
+	if (!exec)
+		return g_strdup ("");
+
+	if (!strchr (exec, ' '))
+		return g_strdup (exec);
+
+	str = g_string_new_len (NULL, strlen (exec));
+
+	str = g_string_append_c (str, '"');
+	for (c = exec; *c != '\0'; c++) {
+		/* FIXME: GKeyFile will add an additional backslach so we'll
+		 * end up with toto\\" instead of toto\"
+		 * We could use g_key_file_set_value(), but then we don't
+		 * benefit from the other escaping that glib is doing...
+		 */
+		if (*c == '"')
+			str = g_string_append (str, "\\\"");
+		else
+			str = g_string_append_c (str, *c);
+	}
+	str = g_string_append_c (str, '"');
+
+	return g_string_free (str, FALSE);
+}
+
 void
 panel_show_help (GdkScreen  *screen,
 		 const char *doc_name,
@@ -350,10 +384,18 @@ panel_error_dialog (GtkWindow  *parent,
 	g_signal_connect (dialog, "event",
 			  G_CALLBACK (panel_dialog_window_event), NULL);
 
-	gtk_window_set_wmclass (GTK_WINDOW (dialog), class, "Panel");
 	if (screen)
 		gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 
+	if (!parent) {
+		gtk_window_set_skip_taskbar_hint (GTK_WINDOW (dialog), FALSE);
+		/* FIXME: We need a title in this case, but we don't know what
+		 * the format should be. Let's put something simple until
+		 * the following bug gets fixed:
+		 * http://bugzilla.gnome.org/show_bug.cgi?id=165132 */
+		gtk_window_set_title (GTK_WINDOW (dialog), _("Error"));
+	}
+
 	gtk_widget_show_all (dialog);
 
 	if (auto_destroy)
@@ -690,39 +732,6 @@ panel_load_icon (GtkIconTheme  *icon_theme,
 	return retval;
 }
 
-GdkPixbuf *
-missing_pixbuf (int size)
-{
-	int i, j;
-	int rowstride;
-	guchar *pixels;
-	GdkPixbuf *pb;
-
-	pb = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
-			     FALSE /* has alpha */,
-			     8 /* bits per sample */,
-			     size, size);
-	pixels = gdk_pixbuf_get_pixels (pb);
-	rowstride = gdk_pixbuf_get_rowstride (pb);
-
-	for (i = 0; i < size; i++) {
-		for (j = 0; j < size; j++) {
-			guchar *p = pixels + i * rowstride + 3 * j;
-			if (i == j || size - i == j) {
-				p[0] = 255;
-				p[1] = 0;
-				p[2] = 0;
-			} else {
-				p[0] = 255;
-				p[1] = 255;
-				p[2] = 255;
-			}
-		}
-	}
-
-	return pb;
-}
-
 static char *
 panel_lock_screen_action_get_command (const char *action)
 {
@@ -840,6 +849,48 @@ panel_make_full_path (const char *dir,
 	return retval;
 }
 
+char *
+panel_make_unique_desktop_path_from_name (const char *dir,
+					  const char *name)
+{
+	int   num = 1;
+	char *path = NULL;
+	char  filename[MAXNAMLEN];
+
+/* g_file_set_contents() use "%s.XXXXXX"
+ * FIXME: waiting for http://bugzilla.gnome.org/show_bug.cgi?id=437977 */
+#define LENGTH_FOR_TMPFILE_EXT 7
+
+	g_snprintf (filename,
+		    sizeof (filename) - strlen (".desktop") - LENGTH_FOR_TMPFILE_EXT,
+		    "%s", name);
+	g_strlcat (filename, ".desktop", sizeof (filename));
+	path = panel_make_full_path (dir, filename);
+	if (!g_file_test (path, G_FILE_TEST_EXISTS))
+		return path;
+	g_free (path);
+
+	while (TRUE) {
+		char *buf;
+
+		buf = g_strdup_printf ("-%d.desktop", num);
+		g_snprintf (filename,
+			    sizeof (filename) - strlen (buf) - LENGTH_FOR_TMPFILE_EXT,
+			    "%s", name);
+		g_strlcat (filename, buf, sizeof (filename));
+		g_free (buf);
+
+		path = panel_make_full_path (dir, filename);
+		if (!g_file_test (path, G_FILE_TEST_EXISTS))
+			return path;
+		g_free (path);
+
+		num++;
+	}
+
+	return NULL;
+}
+
 char *
 panel_make_unique_desktop_uri (const char *dir,
 			       const char *source)
@@ -847,8 +898,6 @@ panel_make_unique_desktop_uri (const char *dir,
 	char     *name, *p;
 	char     *uri;
 	char     *path = NULL;
-	gboolean  exists = TRUE;
-	int       num = 0;
 
 	/* Accept NULL source. Using an emptry string makes our life easier
 	 * than keeping NULL. */
@@ -887,24 +936,11 @@ panel_make_unique_desktop_uri (const char *dir,
 		name = g_strdup (_("file"));
 	}
 
-	while (exists) {
-		char *filename;
-
-		filename = num == 0 ?
-			g_strdup_printf ("%s.desktop", name) :
-			g_strdup_printf ("%s-%d.desktop", name, num);
-
-		g_free (path);
-		path = panel_make_full_path (dir, filename);
-		exists = g_file_test (path, G_FILE_TEST_EXISTS);
-		g_free (filename);
-
-		num++;
-	}
+	path = panel_make_unique_desktop_path_from_name (dir, name);
+	g_free (name);
 
 	uri = gnome_vfs_get_uri_from_local_path (path);
 	g_free (path);
-	g_free (name);
 
 	return uri;
 }
@@ -1242,3 +1278,238 @@ panel_util_key_file_remove_locale_key (GKeyFile    *keyfile,
 		g_key_file_remove_key (keyfile, "Desktop Entry",
 				       key, NULL);
 }
+
+void
+panel_util_key_file_remove_all_locale_key (GKeyFile    *keyfile,
+					   const gchar *key)
+{
+	char **keys;
+	int    key_len;
+	int    i;
+
+	if (!key)
+		return;
+
+	keys = g_key_file_get_keys (keyfile, "Desktop Entry", NULL, NULL);
+	if (!keys)
+		return;
+
+	key_len = strlen (key);
+
+	for (i = 0; keys[i] != NULL; i++) {
+		int len;
+
+		if (strncmp (keys[i], key, key_len))
+			continue;
+
+		len = strlen (keys[i]);
+		if (len == key_len ||
+		    (len > key_len && keys[i][key_len] == '['))
+			g_key_file_remove_key (keyfile, "Desktop Entry",
+					       keys[i], NULL);
+	}
+
+	g_strfreev (keys);
+}
+
+void
+panel_util_key_file_ensure_C_key (GKeyFile   *keyfile,
+				  const char *key)
+{
+	char *C_value;
+	char *buffer;
+
+	/* Make sure we set the "C" locale strings to the terms we set here.
+	 * This is so that if the user logs into another locale they get their
+	 * own description there rather then empty. It is not the C locale
+	 * however, but the user created this entry herself so it's OK */
+	C_value = panel_util_key_file_get_string (keyfile, key);
+	if (C_value == NULL || C_value [0] == '\0') {
+		buffer = panel_util_key_file_get_locale_string (keyfile, key);
+		if (buffer) {
+			panel_util_key_file_set_string (keyfile, key, buffer);
+			g_free (buffer);
+		}
+	}
+	g_free (C_value);
+}
+
+char *
+guess_icon_from_exec (GtkIconTheme *icon_theme,
+		      GKeyFile     *key_file)
+{
+	char *exec;
+	char *icon_name;
+	char *path;
+
+	exec = panel_util_key_file_get_string (key_file, "Exec");
+	if (!exec || !exec [0]) {
+		g_free (exec);
+		return NULL;
+	}
+
+	icon_name = g_path_get_basename (exec);
+	g_free (exec);
+
+	path = panel_find_icon (icon_theme, icon_name, 48);
+	if (!path) {
+		g_free (icon_name);
+		return NULL;
+	}
+
+	return icon_name;
+}
+
+/* This is nautilus_get_vfs_method_display_name() */
+const char *
+panel_util_get_vfs_method_display_name (const char *method)
+{
+	//FIXME: /apps/nautilus/desktop/computer_icon_visible (same for trash and network)
+	if (g_ascii_strcasecmp (method, "computer") == 0 ) {
+		return _("Computer");
+	} else if (g_ascii_strcasecmp (method, "network") == 0 ) {
+		return _("Network");
+	} else if (g_ascii_strcasecmp (method, "fonts") == 0 ) {
+		return _("Fonts");
+	} else if (g_ascii_strcasecmp (method, "themes") == 0 ) {
+		return _("Themes");
+	} else if (g_ascii_strcasecmp (method, "burn") == 0 ) {
+		return _("CD/DVD Creator");
+	} else if (g_ascii_strcasecmp (method, "smb") == 0 ) {
+		return _("Windows Network");
+	} else if (g_ascii_strcasecmp (method, "dns-sd") == 0 ) {
+		/* translators: this is the title of the "dns-sd:///" location */
+		return _("Services in");
+	}
+	return NULL;
+}
+
+static const char *
+panel_util_get_icon_for_uri_method (const char *uri)
+{
+	if (g_str_has_prefix (uri, "computer:")) {
+		return PANEL_ICON_COMPUTER;
+	} else if (g_str_has_prefix (uri, "network:")) {
+		return PANEL_ICON_NETWORK;
+	} else if (g_str_has_prefix (uri, "fonts:")) {
+		return PANEL_ICON_FONTS;
+	} else if (g_str_has_prefix (uri, "themes:")) {
+		return PANEL_ICON_THEME;
+	} else if (g_str_has_prefix (uri, "burn:")) {
+		return PANEL_ICON_BURNER;
+	} else if (g_str_has_prefix (uri, "smb:")) {
+		return PANEL_ICON_NETWORK;
+	} else if (g_str_has_prefix (uri, "dns-sd:")) {
+		return PANEL_ICON_NETWORK;
+	} else if (g_str_has_prefix (uri, "trash:")) {
+		//FIXME: we should look if the trash is empty or not
+		return PANEL_ICON_TRASH;
+	} else if (g_str_has_prefix (uri, "x-nautilus-search:")) {
+		return PANEL_ICON_SEARCHTOOL;
+	} else
+		return NULL;
+}
+
+/* This is based on nautilus_compute_title_for_uri() and
+ * nautilus_file_get_display_name_nocopy() */
+char *
+panel_util_get_label_for_uri (const char *text_uri)
+{
+	GnomeVFSURI *uri;
+	const char  *hostname;
+	const char  *method;
+	char        *displayname;
+	char        *label;
+	char        *buffer;
+
+	//FIXME: we're not handling $HOME and $Desktop
+	hostname = NULL;
+	label = NULL;
+
+	//FIXME: nautilus uses nautilus_query_to_readable_string() to have a nice name
+	if (g_str_has_prefix (text_uri, "x-nautilus-search:"))
+		return g_strdup (_("Search"));
+
+	if (g_str_has_prefix (text_uri, "trash:"))
+		return g_strdup (_("Trash"));
+
+	if (g_str_has_prefix (text_uri, "file:")) {
+		buffer = gnome_vfs_get_local_path_from_uri (text_uri);
+		if (!buffer)
+			return NULL;
+
+		label = g_filename_display_basename (buffer);
+		g_free (buffer);
+
+		return label;
+	}
+
+	uri = gnome_vfs_uri_new (text_uri);
+	if (uri) {
+		char *short_name;
+
+		hostname = gnome_vfs_uri_get_host_name (uri);
+
+		buffer = gnome_vfs_uri_extract_short_path_name (uri);
+		short_name = gnome_vfs_unescape_string_for_display (buffer);
+		g_free (buffer);
+
+		if (strcmp (short_name, GNOME_VFS_URI_PATH_STR) != 0) {
+			displayname = short_name;
+		} else {
+			g_free (short_name);
+			method = panel_util_get_vfs_method_display_name (uri->method_string);
+			if (method == NULL)
+				method = uri->method_string;
+
+			buffer = gnome_vfs_uri_extract_short_name (uri);
+			short_name = gnome_vfs_unescape_string_for_display (buffer);
+			g_free (buffer);
+
+			if (short_name == NULL ||
+			    strcmp (short_name, GNOME_VFS_URI_PATH_STR) == 0) {
+				displayname = g_strdup (method);
+			} else {
+				displayname = g_strdup_printf ("%s: %s",
+							       method,
+							       short_name);
+			}
+			g_free (short_name);
+		}
+	} else {
+		displayname = gnome_vfs_format_uri_for_display (text_uri);
+	}
+
+	if (hostname) {
+		/* Translators: the first string is a path and the second
+		 * string is a hostname. nautilus contains the same string to
+		 * translate. */
+		label = g_strdup_printf (_("%1$s on %2$s"), displayname, hostname);
+		g_free (displayname);
+	} else {
+		label = displayname;
+	}
+
+	if (uri)
+		gnome_vfs_uri_unref (uri);
+
+	return label;
+}
+
+char *
+panel_util_get_icon_for_uri (const char *text_uri)
+{
+	const char *icon;
+
+	icon = panel_util_get_icon_for_uri_method (text_uri);
+	if (icon)
+		return g_strdup (icon);
+
+	if (!g_str_has_prefix (text_uri, "file:"))
+		return NULL;
+
+	return gnome_icon_lookup_sync (gtk_icon_theme_get_default (),
+				       NULL, text_uri, NULL,
+				       GNOME_ICON_LOOKUP_FLAGS_NONE,
+				       GNOME_ICON_LOOKUP_RESULT_FLAGS_NONE);
+}
